/** 
 * Copyright (c) SpaceToad, 2011
 * http://www.mod-buildcraft.com
 * 
 * BuildCraft is distributed under the terms of the Minecraft Mod Public 
 * License 1.0, or MMPL. Please check the contents of the license located in
 * http://www.mod-buildcraft.com/MMPL-1.0.txt
 */

package net.minecraft.src.buildcraft.api;

import java.util.LinkedList;

import net.minecraft.src.Block;
import net.minecraft.src.BlockContainer;
import net.minecraft.src.ItemStack;
import net.minecraft.src.TileEntity;
import net.minecraft.src.World;

/**
 * This class allow to specify specific behavior for blocks stored in 
 * blueprints:
 * 
 * - what items needs to be used to create that block
 * - how the block has to be built on the world
 * - how to rotate the block
 * - what extra data to store / load in the blueprint
 * 
 * Default implementations of this can be seen in the package 
 * net.minecraft.src.buildcraft.api.bptblocks. The class BptBlockUtils provide
 * some additional utilities.
 * 
 * Blueprints perform "id translation" in case the block ids between a 
 * blueprint and the world installation are different. In order to translate
 * block ids, blocks needs to be uniquely identified. By default, this 
 * identification is done by:
 * 
 * - the block simple class name
 * - the tile simple class name (if any)
 * - the block name
 * 
 * In certain circumstances, the above is not enough (e.g. if several blocks
 * share the same class and the same name, with no tile). In this case, 
 * additional data may be provided by children of this class:
 * 
 * - mod name
 * - custom signature
 * 
 * At blueprint load time, BuildCraft will check that each block id of the
 * blueprint corresponds to the block id in the installation. If not, it will
 * perform a search through the block list, and upon matching signature, it
 * will translate all blocks ids of the blueprint to the installation ones. If
 * no such block id is found, BuildCraft will assume that the block is not
 * installed and will not load the blueprint.
 */
public class BptBlock {
	
	public final int blockId;
	
	public BptBlock (int blockId) {
		this.blockId = blockId;
		
		BuildCraftAPI.blockBptProps [blockId] = this;
	}
	
	
	/**
	 * Returns the requirements needed to build this block. When the 
	 * requirements are met, they will be removed all at once from the 
	 * builder, before calling buildBlock.
	 */
	public void addRequirements(BptSlotInfo slot, LinkedList <ItemStack> requirements) {
		requirements.add (new ItemStack (slot.blockId, 1, slot.meta));
	}

	/**
	 * Return true if the block on the world correspond to the block stored
	 * in the blueprint at the location given by the slot. By default, this
	 * subprogram is permissive and doesn't take into account metadata.
	 */
	public boolean isValid(World world, BptSlotInfo slot) {
		return slot.blockId == world.getBlockId(slot.x, slot.y, slot.z);
	}

	/**
	 * Perform a 90 degree rotation to the slot.
	 */
	public void rotateLeft(BptSlotInfo slot) {

	}

	/**
	 * Places the block in the world, at the location specified in the slot.
	 */
	public void buildBlock(BptSlotInfo slot, World world) {
		//  Meta needs to be specified twice, depending on the block behavior
		world.setBlockAndMetadataWithNotify(slot.x, slot.y, slot.z, slot.blockId, slot.meta);
		world.setBlockMetadataWithNotify(slot.x, slot.y, slot.z, slot.meta);
		
		if (Block.blocksList [slot.blockId] instanceof BlockContainer) {
			TileEntity tile = world.getBlockTileEntity(slot.x, slot.y, slot.z);
			
			slot.cpt.setInteger("x", slot.x);
			slot.cpt.setInteger("y", slot.y);
			slot.cpt.setInteger("z", slot.z);
			
			if (tile != null) {
				tile.readFromNBT(slot.cpt);
			}
		}
	}
	
	/**
	 * Return true if the block should not be placed to the world. Requirements
	 * will not be asked on such a block, and building will not be called.
	 */
	public boolean ignoreBuilding(BptSlotInfo slot) {
		return false;
	}

	/**
	 * Initializes a slot from the blueprint according to an objet placed on 
	 * {x, y, z} on the world. This typically means adding entries in slot.cpt.
	 * Note that "id" and "meta" will be set automatically, corresponding to
	 * the block id and meta.
	 * 
	 * By default, if the block is a BlockContainer, tile information will be
	 * to save / load the block.
	 */
	public void initializeFromWorld(BptSlotInfo slot, World worldObj, int x, int y, int z) {
		if (Block.blocksList [slot.blockId] instanceof BlockContainer) {
			TileEntity tile = worldObj.getBlockTileEntity(x, y, z);
			
			if (tile != null) {
				tile.writeToNBT(slot.cpt);
			}
		}
	}

	/**
	 * Called on a block when the blueprint has finished to place all the 
	 * blocks. This may be useful to adjust variable depending on surrounding
	 * blocks that may not be there already at initial building.
	 */
	public void postProcessing(BptSlotInfo slot, World world) {

	}	
	
	/**
	 * By default, block class name, block tile name and block name are used
	 * to define block signature. Overriding this subprogram may allow to
	 * replace some of these with stars, specify the mod that this block kind
	 * is coming from or add custom data to the signature.
	 */
	public BlockSignature getSignature (Block block) {
		BlockSignature sig = new BlockSignature();
		
		if (block.blockID > 122) {
			sig.blockClassName = block.getClass().getSimpleName();

			if (block instanceof BlockContainer) {
				TileEntity tile = ((BlockContainer) block).getBlockEntity();

				if (tile != null) {
					sig.tileClassName = tile.getClass().getSimpleName();
				}
			}
		}
		
		sig.blockName = block.getBlockName();
		sig.replaceNullWithStar();
		
		return sig;
	}
	
	/**
	 * By default, block name, block and tile classes, mod name and custom 
	 * signature are matched to verify if a blueprint block corresponds to the
	 * installation block - except for the default blocks which don't check 
	 * for classes.
	 * For any value, * means match with anything.
	 * For compatibilty and evolution reasons, mods may want to write a 
	 * different policy, allowing to migrate one format to the
	 * other.
	 */
	public boolean match (Block block, BlockSignature sig) {
		if (block == null) {
			return false;
		}
		
		BlockSignature inst = BuildCraftAPI.getBlockSignature(block);
		
		return starMatch (sig.blockName, inst.blockName)
				&& starMatch (sig.blockClassName, inst.blockClassName)
				&& starMatch (sig.tileClassName, inst.tileClassName)
				&& starMatch (sig.customField, inst.customField)
				&& starMatch (sig.mod, inst.mod);
	}
	
	private boolean starMatch (String s1, String s2) {
		return s1.equals("*") || s2.equals("*") || s1.equals(s2);
	}
}
